#include <stdio.h>
#include <SDL/SDL.h>
#include <SDL/SDL_thread.h>
#include <SDL/SDL_syswm.h>
#include <GL/gl.h>
#include <GL/glx.h>

#include "Shared.h"
#include "../Shared/CObeliskException.h"
#include "CApplication.h"
#include "CThreadWrapper.h"

using namespace Obelisk;



CThreadWrapper::CThreadWrapper(CApplication* aApplication, IThread* aThread)
{
	mApplication = aApplication;
	mThread = aThread;
	mStoppedMutex = SDL_CreateMutex();
	mStopped = false;
	mIOStopped = false;
}



CThreadWrapper::~CThreadWrapper()
{
	#ifndef NO_THREADS
		SDL_WaitThread(mSDLThread, NULL);
	#endif
	SDL_DestroyMutex(mStoppedMutex);
	delete mThread;
}



void CThreadWrapper::Run()
{
	#ifndef NO_THREADS
		mSDLThread = SDL_CreateThread(ThreadProc, this);
	#else
		mThread->Run(this);
		SDL_Event Event;
		Event.type = SDL_USEREVENT;
		Event.user.code = 1;
		Event.user.data1 = mThread;
		Event.user.data2 = NULL;
		int Result = SDL_PushEvent(&Event);
		Assert(Result == 0)
	#endif
}



int CThreadWrapper::ThreadProc(void* aData)
{
	((CThreadWrapper*)aData)->ThreadMain();
	return 0;
}



void CThreadWrapper::ThreadMain()
{
	try
	{
		mThread->Run(this);
		if (glXGetCurrentContext() != NULL)
		{
			bool MCResult = glXMakeCurrent(mApplication->GetDisplay(), None, NULL);
			Assert(MCResult)
		}

		SDL_Event Event;
		Event.type = SDL_USEREVENT;
		Event.user.code = 1;
		Event.user.data1 = mThread;
		Event.user.data2 = NULL;
		int Result = SDL_PushEvent(&Event);
		Assert(Result == 0)
	}
	catch (CObeliskException& e)
	{
		char *Message = new char [e.GetMessage().GetSize() + 1];
		strcpy(Message, e.GetMessage().GetData());
		SDL_Event Event;
		Event.type = SDL_USEREVENT;
		Event.user.code = 3;
		Event.user.data1 = Message;
		Event.user.data2 = NULL;
		SDL_PushEvent(&Event);
	}
	catch (CException&)
	{
		SDL_Event Event;
		Event.type = SDL_USEREVENT;
		Event.user.code = 2;
		Event.user.data1 = NULL;
		Event.user.data2 = NULL;
		SDL_PushEvent(&Event);
	}
}



void CThreadWrapper::Stop()
{
	int Result = SDL_mutexP(mStoppedMutex);
	Assert(Result == 0)
	mStopped = true;
	Result = SDL_mutexV(mStoppedMutex);
	Assert(Result == 0)
}



bool CThreadWrapper::IsStopped()
{
	int Result = SDL_mutexP(mStoppedMutex);
	Assert(Result == 0)
	bool Stopped = mStopped;
	Result = SDL_mutexV(mStoppedMutex);
	Assert(Result == 0)
	if (Stopped)
	{
		mIOStopped = true;
		return true;
	}
	return false;
}



CString CThreadWrapper::IntToString(int aNumber, int aLeading0s)
{
	return mApplication->IntToString(aNumber, aLeading0s);
}



void CThreadWrapper::NotifyWindow(int aMessage, int aData)
{
	mApplication->NotifyWindow(aMessage, aData);
}



double CThreadWrapper::Random()
{
	return mApplication->Random();
}



int CThreadWrapper::Random(int aMin, int aMax)
{
	return mApplication->Random(aMin, aMax);
}



void CThreadWrapper::AttachRenderingContext()
{
	mApplication->AttachRenderingContext();
}



bool CThreadWrapper::LoadTexture(CString aName, bool aTransparent)
{
	if (mIOStopped)
		throw CException("Thread IO not allowed");
	mApplication->LoadTexture(aName, aTransparent);
	return !IsStopped();
}



bool CThreadWrapper::LoadSound(CString aName)
{
	if (mIOStopped)
		throw CException("Thread IO not allowed");
	mApplication->LoadSound(aName);
	return !IsStopped();
}



IReader* CThreadWrapper::OpenDataFile(CString aFileName)
{
	if (mIOStopped)
		throw CException("Thread IO not allowed");
	return IsStopped() ? NULL : mApplication->OpenDataFile(aFileName);
}



IReader* CThreadWrapper::ReadUserFile(CString aFileName)
{
	if (mIOStopped)
		throw CException("Thread IO not allowed");
	return IsStopped() ? NULL : mApplication->ReadUserFile(aFileName);
}



IWriter* CThreadWrapper::WriteUserFile(CString aFileName)
{
	if (mIOStopped)
		throw CException("Thread IO not allowed");
	return IsStopped() ? NULL : mApplication->WriteUserFile(aFileName);
}
